@@ -207,7 +207,7 @@ static void on_event(MirSurface *surface, const MirEvent *event, void *context)
         static size_t max_fingers = 0;
         static float max_pressure = 1.0f;
         
-        // FIXME: https://bugs.launchpad.net/mir/+bug/1197108
+        // FIXME: https://bugs.launchpad.net/mir/+bug/1311699
         MirMotionAction masked_action = event->motion.action & ~0xff00;
 
         if (masked_action == mir_motion_action_up)
@@ -286,13 +286,14 @@ static const MirDisplayOutput *find_active_output(
 
 int main(int argc, char *argv[])
 {
-    static const Color background = {0x40, 0x40, 0x40, 0xff};
+    static const Color background = {180, 180, 150, 255};
     MirConnection *conn;
     MirSurfaceParameters parm;
     MirSurface *surf;
     MirGraphicsRegion canvas;
     MirEventDelegate delegate = {&on_event, &canvas};
     unsigned int f;
+    int swap_interval = 0;
 
     char *mir_socket = NULL;
 
@@ -311,6 +312,9 @@ int main(int argc, char *argv[])
                 case 'm':
                     mir_socket = argv[++i];
                     break;
+                case 'w':
+                    swap_interval = 1;
+                    break;
                 case 'h':
                 default:
                     help = 1;
@@ -327,6 +331,7 @@ int main(int argc, char *argv[])
                 printf("Usage: %s [<options>]\n"
                        "  -h               Show this help text\n"
                        "  -m socket        Mir server socket\n"
+                       "  -w               Wait for vblank (don't drop frames)\n"
                        , argv[0]);
                 return 0;
             }
@@ -385,6 +390,7 @@ int main(int argc, char *argv[])
     surf = mir_connection_create_surface_sync(conn, &parm);
     if (surf != NULL)
     {
+        mir_surface_set_swapinterval(surf, swap_interval);
         mir_surface_set_event_handler(surf, &delegate);
     
         canvas.width = parm.width;
@@ -397,6 +403,7 @@ int main(int argc, char *argv[])
         {
             signal(SIGINT, shutdown);
             signal(SIGTERM, shutdown);
+            signal(SIGHUP, shutdown);
         
             clear_region(&canvas, &background);
             redraw(surf, &canvas);
