@@ -22,6 +22,9 @@
 
 /* This header defines debug interfaces that aren't expected to be generally useful
  * and do not have the same API-stability guarantees that the main API has */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * Return the ID of a surface (only useful for debug output).
@@ -40,4 +43,8 @@ int mir_debug_surface_id(MirSurface *surface);
  */
 uint32_t mir_debug_surface_current_buffer_id(MirSurface *surface);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* MIR_CLIENT_LIBRARY_DEBUG_H */
