@@ -18,6 +18,8 @@
 #ifndef MIR_GRAPHICS_NATIVE_PLATFORM_H_
 #define MIR_GRAPHICS_NATIVE_PLATFORM_H_
 
+#include "mir/graphics/platform.h"
+#include "mir/graphics/platform_ipc_operations.h"
 #include <memory>
 #include <functional>
 
@@ -33,8 +35,9 @@ class GraphicBufferAllocator;
 class BufferInitializer;
 class PlatformIPCPackage;
 class InternalClient;
-class BufferIPCPacker;
+class BufferIpcMessage;
 class Buffer;
+class BufferWriter;
 class DisplayReport;
 class NestedContext;
 
@@ -48,11 +51,16 @@ public:
     virtual std::shared_ptr<GraphicBufferAllocator> create_buffer_allocator(
         std::shared_ptr<BufferInitializer> const& buffer_initializer) = 0;
 
-    virtual std::shared_ptr<PlatformIPCPackage> get_ipc_package() = 0;
+    virtual std::shared_ptr<PlatformIPCPackage> connection_ipc_package() = 0;
 
     virtual std::shared_ptr<InternalClient> create_internal_client() = 0;
 
-    virtual void fill_ipc_package(BufferIPCPacker* packer, Buffer const* buffer) const = 0;
+    virtual void fill_buffer_package(
+        BufferIpcMessage* message,
+        Buffer const* buffer,
+        BufferIpcMsgType msg_type) const = 0;
+
+    virtual std::shared_ptr<BufferWriter> make_buffer_writer() = 0;
 
     virtual ~NativePlatform() = default;
     NativePlatform(NativePlatform const&) = delete;
