@@ -27,6 +27,8 @@
 
 namespace mir
 {
+class EmergencyCleanupRegistry;
+
 namespace frontend
 {
 class Surface;
@@ -40,16 +42,17 @@ class Option;
 /// the graphics environment.
 namespace graphics
 {
-class BufferIPCPacker;
 class Buffer;
 class Display;
-struct PlatformIPCPackage;
 class BufferInitializer;
 class InternalClient;
 class DisplayReport;
 class DisplayConfigurationPolicy;
 class GraphicBufferAllocator;
 class GLConfig;
+class GLProgramFactory;
+class PlatformIpcOperations;
+class BufferWriter;
 
 /**
  * \defgroup platform_enablement Mir platform enablement
@@ -78,31 +81,22 @@ public:
 
     virtual std::shared_ptr<GraphicBufferAllocator> create_buffer_allocator(
         std::shared_ptr<BufferInitializer> const& buffer_initializer) = 0;
+    
+    virtual std::shared_ptr<BufferWriter> make_buffer_writer() = 0;
 
     /**
      * Creates the display subsystem.
      */
     virtual std::shared_ptr<Display> create_display(
         std::shared_ptr<DisplayConfigurationPolicy> const& initial_conf_policy,
+        std::shared_ptr<GLProgramFactory> const& gl_program_factory,
         std::shared_ptr<GLConfig> const& gl_config) = 0;
 
     /**
-     * Gets the IPC package for the platform.
-     *
-     * The IPC package will be sent to clients when they connect.
+     * Creates an object capable of doing platform specific processing of buffers
+     * before they are sent or after they are recieved accross IPC
      */
-    virtual std::shared_ptr<PlatformIPCPackage> get_ipc_package() = 0;
-
-    /**
-     * Fills the IPC package for a buffer.
-     *
-     * The Buffer IPC package will be sent to clients when receiving a buffer.
-     * The implementation must use the provided packer object to perform the packing.
-     *
-     * \param [in] packer the object providing the packing functionality
-     * \param [in] buffer the buffer to fill the IPC package for
-     */
-    virtual void fill_ipc_package(BufferIPCPacker* packer, Buffer const* buffer) const = 0;
+    virtual std::shared_ptr<PlatformIpcOperations> make_ipc_operations() const = 0;
 
     /**
      * Creates the in-process client support object.
@@ -114,14 +108,21 @@ public:
  * Function prototype used to return a new graphics platform.
  *
  * \param [in] options options to use for this platform
+ * \param [in] emergency_cleanup_registry object to register emergency shutdown handlers with
  * \param [in] report the object to use to report interesting events from the display subsystem
  *
  * This factory function needs to be implemented by each platform.
  *
  * \ingroup platform_enablement
  */
-extern "C" typedef std::shared_ptr<Platform>(*CreatePlatform)(std::shared_ptr<options::Option> const& options, std::shared_ptr<DisplayReport> const& report);
-extern "C" std::shared_ptr<Platform> create_platform (std::shared_ptr<options::Option> const& options, std::shared_ptr<DisplayReport> const& report);
+extern "C" typedef std::shared_ptr<Platform>(*CreatePlatform)(
+    std::shared_ptr<options::Option> const& options,
+    std::shared_ptr<EmergencyCleanupRegistry> const& emergency_cleanup_registry,
+    std::shared_ptr<DisplayReport> const& report);
+extern "C" std::shared_ptr<Platform> create_platform(
+    std::shared_ptr<options::Option> const& options,
+    std::shared_ptr<EmergencyCleanupRegistry> const& emergency_cleanup_registry,
+    std::shared_ptr<DisplayReport> const& report);
 extern "C" typedef void(*AddPlatformOptions)(
     boost::program_options::options_description& config);
 extern "C" void add_platform_options(
