@@ -1,5 +1,5 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Authored by: Robert Carr <robert.carr@canonical.com>
+ * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #ifndef MIR_INPUT_INPUT_CONFIGURATION_H_
@@ -23,36 +23,24 @@
 
 namespace mir
 {
-namespace scene
-{
-class InputRegistrar;
-}
-namespace shell
-{
-class InputTargeter;
-}
 namespace input
 {
-class InputTargets;
 class InputManager;
 
 class InputConfiguration
 {
 public:
-    virtual ~InputConfiguration() {}
+    virtual ~InputConfiguration() = default;
 
-    virtual std::shared_ptr<scene::InputRegistrar> the_input_registrar() = 0;
-    virtual std::shared_ptr<shell::InputTargeter> the_input_targeter() = 0;
-    virtual std::shared_ptr<input::InputManager> the_input_manager() = 0;
-
-    virtual void set_input_targets(std::shared_ptr<input::InputTargets> const& targets) = 0;
+    virtual std::shared_ptr<InputManager> the_input_manager() = 0;
 
 protected:
     InputConfiguration() = default;
     InputConfiguration(InputConfiguration const&) = delete;
     InputConfiguration& operator=(InputConfiguration const&) = delete;
 };
-}
-} // namespace mir
 
-#endif // MIR_INPUT_INPUT_CONFIGURATION_H_
+}
+}
+
+#endif
