@@ -29,6 +29,7 @@ namespace client
 class SurfaceMap;
 class DisplayConfiguration;
 class LifecycleControl;
+class EventSink;
 
 namespace rpc
 {
@@ -39,7 +40,8 @@ make_rpc_channel(std::string const& name,
                  std::shared_ptr<SurfaceMap> const& map,
                  std::shared_ptr<DisplayConfiguration> const& disp_conf,
                  std::shared_ptr<RpcReport> const& rpc_report,
-                 std::shared_ptr<LifecycleControl> const& lifecycle_control);
+                 std::shared_ptr<LifecycleControl> const& lifecycle_control,
+                 std::shared_ptr<EventSink> const& event_distributor);
 }
 }
 }
