@@ -17,6 +17,7 @@
  */
 
 #include "default_display_buffer_compositor_factory.h"
+#include "mir/compositor/destination_alpha.h"
 #include "mir/compositor/renderer_factory.h"
 #include "mir/compositor/renderer.h"
 #include "mir/graphics/display_buffer.h"
@@ -40,7 +41,9 @@ std::unique_ptr<mc::DisplayBufferCompositor>
 mc::DefaultDisplayBufferCompositorFactory::create_compositor_for(
     graphics::DisplayBuffer& display_buffer)
 {
-    auto renderer = renderer_factory->create_renderer_for(display_buffer.view_area());
+    auto dest_alpha = display_buffer.uses_alpha() ?
+        DestinationAlpha::generate_from_source : DestinationAlpha::opaque;
+    auto renderer = renderer_factory->create_renderer_for(display_buffer.view_area(), dest_alpha);
     auto raw = new DefaultDisplayBufferCompositor{display_buffer, scene,
                                                   std::move(renderer),
                                                   report};
