@@ -19,8 +19,9 @@
 #ifndef MIR_FRONTEND_PROTOBUF_BUFFER_PACKER_H_
 #define MIR_FRONTEND_PROTOBUF_BUFFER_PACKER_H_
 
-#include "mir/graphics/buffer_ipc_packer.h"
+#include "mir/graphics/buffer_ipc_message.h"
 #include "mir_protobuf.pb.h"
+#include <memory>
 
 namespace mir
 {
@@ -36,16 +37,20 @@ namespace detail
 void pack_protobuf_display_configuration(protobuf::DisplayConfiguration& protobuf_config,
                                          graphics::DisplayConfiguration const& display_config);
 
-class ProtobufBufferPacker : public graphics::BufferIPCPacker
+class ProtobufBufferPacker : public graphics::BufferIpcMessage
 {
 public:
     ProtobufBufferPacker(protobuf::Buffer*);
-    void pack_fd(int);
+    void pack_fd(Fd const&);
     void pack_data(int);
     void pack_stride(geometry::Stride);
     void pack_flags(unsigned int);
     void pack_size(geometry::Size const& size);
+
+    std::vector<Fd> fds();
+    std::vector<int> data();
 private:
+    std::vector<mir::Fd> fds_;
     protobuf::Buffer* buffer_response;
 };
 
