@@ -45,7 +45,7 @@ auto mf::as_internal_surface(std::shared_ptr<Surface> const& surface)
         {
             surface->swap_buffers_blocking(buffer);
         }
-        virtual mir::geometry::Size size() const { return surface->size(); }
+        virtual mir::geometry::Size size() const { return surface->client_size(); }
         virtual MirPixelFormat pixel_format() const { return surface->pixel_format(); }
 
         std::shared_ptr<Surface> const surface;
