@@ -19,15 +19,15 @@
 #include "mir_test_framework/input_testing_server_configuration.h"
 
 #include "mir/input/input_channel.h"
-#include "mir/scene/input_registrar.h"
 #include "mir/input/surface.h"
-#include "mir/shell/surface_creation_parameters.h"
+#include "mir/scene/surface_creation_parameters.h"
 #include "mir/frontend/shell.h"
 #include "mir/frontend/session.h"
 #include "mir/input/composite_event_filter.h"
 
 #include "mir_test/fake_event_hub.h"
 #include "mir_test/fake_event_hub_input_configuration.h"
+#include "mir_test/wait_condition.h"
 
 #include <boost/throw_exception.hpp>
 
@@ -35,10 +35,12 @@
 #include <stdexcept>
 
 namespace mtf = mir_test_framework;
+namespace mt = mir::test;
 
 namespace mf = mir::frontend;
 namespace mg = mir::graphics;
 namespace mi = mir::input;
+namespace ms = mir::shell;
 namespace mia = mi::android;
 namespace geom = mir::geometry;
 namespace mtd = mir::test::doubles;
@@ -47,9 +49,29 @@ mtf::InputTestingServerConfiguration::InputTestingServerConfiguration()
 {
 }
 
-void mtf::InputTestingServerConfiguration::exec()
+mtf::InputTestingServerConfiguration::InputTestingServerConfiguration(
+    std::vector<geom::Rectangle> const& display_rects) :
+    TestingServerConfiguration(display_rects)
 {
-    input_injection_thread = std::thread(std::mem_fn(&mtf::InputTestingServerConfiguration::inject_input), this);
+}
+
+void mtf::InputTestingServerConfiguration::on_start()
+{
+    auto const start_input_injection = std::make_shared<mt::WaitCondition>();
+
+    input_injection_thread = std::thread{
+        [this, start_input_injection]
+        {
+            // We need to wait for the 'input_injection_thread' variable to be
+            // assigned to before starting. Otherwise we may end up calling
+            // on_exit() before assignment and try to join an unjoinable thread.
+            start_input_injection->wait_for_at_most_seconds(3);
+            if (!start_input_injection->woken())
+                BOOST_THROW_EXCEPTION(std::runtime_error("Input injection thread start signal timed out"));
+            inject_input();
+        }};
+
+    start_input_injection->wake_up_everyone();
 }
 
 void mtf::InputTestingServerConfiguration::on_exit()
@@ -57,21 +79,36 @@ void mtf::InputTestingServerConfiguration::on_exit()
     input_injection_thread.join();
 }
 
+std::shared_ptr<ms::InputTargeter> mtf::InputTestingServerConfiguration::the_input_targeter()
+{
+    return DefaultServerConfiguration::the_input_targeter();
+}
+
+std::shared_ptr<mi::InputDispatcher> mtf::InputTestingServerConfiguration::the_input_dispatcher()
+{
+    return DefaultServerConfiguration::the_input_dispatcher();
+}
+
+std::shared_ptr<mi::InputSender> mtf::InputTestingServerConfiguration::the_input_sender()
+{
+    return DefaultServerConfiguration::the_input_sender();
+}
+
 std::shared_ptr<mi::InputConfiguration> mtf::InputTestingServerConfiguration::the_input_configuration()
 {
     if (!input_configuration)
     {
-        std::shared_ptr<mi::CursorListener> null_cursor_listener{nullptr};
-
         input_configuration = std::make_shared<mtd::FakeEventHubInputConfiguration>(
-            the_composite_event_filter(),
+            the_input_dispatcher(),
             the_input_region(),
-            null_cursor_listener,
+            the_cursor_listener(),
+            the_touch_visualizer(),
             the_input_report());
         fake_event_hub = input_configuration->the_fake_event_hub();
 
         fake_event_hub->synthesize_builtin_keyboard_added();
         fake_event_hub->synthesize_builtin_cursor_added();
+        fake_event_hub->synthesize_usb_touchscreen_added();
         fake_event_hub->synthesize_device_scan_complete();
     }
 
