@@ -17,15 +17,22 @@
  */
 
 #include "src/server/compositor/multi_threaded_compositor.h"
+#include "src/server/report/null_report_factory.h"
+
 #include "mir/compositor/display_buffer_compositor.h"
 #include "mir/compositor/scene.h"
 #include "mir/compositor/display_buffer_compositor_factory.h"
-#include "src/server/report/null_report_factory.h"
+#include "mir/scene/observer.h"
+
+#include "mir_test/current_thread_name.h"
 #include "mir_test_doubles/null_display.h"
 #include "mir_test_doubles/null_display_buffer.h"
 #include "mir_test_doubles/mock_display_buffer.h"
 #include "mir_test_doubles/mock_compositor_report.h"
 #include "mir_test_doubles/mock_scene.h"
+#include "mir_test_doubles/stub_scene.h"
+
+#include <boost/throw_exception.hpp>
 
 #include <unordered_map>
 #include <unordered_set>
@@ -38,9 +45,11 @@
 
 namespace mc = mir::compositor;
 namespace mg = mir::graphics;
+namespace ms = mir::scene;
+namespace mr = mir::report;
 namespace geom = mir::geometry;
 namespace mtd = mir::test::doubles;
-namespace mr = mir::report;
+namespace mt = mir::test;
 
 namespace
 {
@@ -81,43 +90,48 @@ private:
     std::vector<testing::NiceMock<mtd::MockDisplayBuffer>> buffers;
 };
 
-class StubScene : public mc::Scene
+class StubScene : public mtd::StubScene
 {
 public:
-    StubScene() : callback{[]{}} {}
-
-    mg::RenderableList generate_renderable_list() const
+    void add_observer(std::shared_ptr<ms::Observer> const& observer_)
     {
-        return mg::RenderableList{};
+        std::lock_guard<std::mutex> lock{observer_mutex};
+
+        if (throw_on_add_observer_)
+            BOOST_THROW_EXCEPTION(std::runtime_error(""));
+
+        assert(!observer);
+        observer = observer_;
     }
 
-    void for_each_if(mc::FilterForScene&, mc::OperatorForScene&)
+    void remove_observer(std::weak_ptr<ms::Observer> const& /* observer */)
     {
-    }
-
-    void set_change_callback(std::function<void()> const& f)
-    {
-        std::lock_guard<std::mutex> lock{callback_mutex};
-        assert(f);
-        callback = f;
+        std::lock_guard<std::mutex> lock{observer_mutex};
+        observer.reset();
     }
 
     void emit_change_event()
     {
         {
-            std::lock_guard<std::mutex> lock{callback_mutex};
-            callback();
+            std::lock_guard<std::mutex> lock{observer_mutex};
+            
+            // Any old event will do.
+            if (observer)
+                observer->surfaces_reordered();
         }
         /* Reduce run-time under valgrind */
         std::this_thread::yield();
     }
 
-    void lock() {}
-    void unlock() {}
+    void throw_on_add_observer(bool flag)
+    {
+        throw_on_add_observer_ = flag;
+    }
 
 private:
-    std::function<void()> callback;
-    std::mutex callback_mutex;
+    std::mutex observer_mutex;
+    std::shared_ptr<ms::Observer> observer;
+    bool throw_on_add_observer_;
 };
 
 class RecordingDisplayBufferCompositor : public mc::DisplayBufferCompositor
@@ -128,12 +142,11 @@ public:
     {
     }
 
-    bool composite()
+    void composite()
     {
         mark_render_buffer();
         /* Reduce run-time under valgrind */
         std::this_thread::yield();
-        return false;
     }
 
 private:
@@ -165,10 +178,8 @@ public:
         records[&display_buffer].second.insert(std::this_thread::get_id());
     }
 
-    bool enough_records_gathered(unsigned int nbuffers)
+    bool enough_records_gathered(unsigned int nbuffers, unsigned int min_record_count = 1000)
     {
-        static unsigned int const min_record_count{1000};
-
         std::lock_guard<std::mutex> lk{m};
 
         if (records.size() < nbuffers)
@@ -247,12 +258,11 @@ public:
     {
     }
 
-    bool composite()
+    void composite()
     {
         fake_surface_update();
         /* Reduce run-time under valgrind */
         std::this_thread::yield();
-        return false;
     }
 
 private:
@@ -297,7 +307,7 @@ public:
     {
         struct NullDisplayBufferCompositor : mc::DisplayBufferCompositor
         {
-            bool composite() { return false; }
+            void composite() {}
         };
 
         auto raw = new NullDisplayBufferCompositor{};
@@ -305,6 +315,30 @@ public:
     }
 };
 
+class ThreadNameDisplayBufferCompositorFactory : public mc::DisplayBufferCompositorFactory
+{
+public:
+    std::unique_ptr<mc::DisplayBufferCompositor> create_compositor_for(mg::DisplayBuffer&)
+    {
+        auto raw = new RecordingDisplayBufferCompositor{
+            [this]()
+            {
+                std::lock_guard<std::mutex> lock{thread_names_mutex};
+                thread_names.emplace_back(mt::current_thread_name());
+            }};
+        return std::unique_ptr<RecordingDisplayBufferCompositor>(raw);
+    }
+
+    size_t num_thread_names_gathered()
+    {
+        std::lock_guard<std::mutex> lock{thread_names_mutex};
+        return thread_names.size();
+    }
+
+    std::mutex thread_names_mutex;
+    std::vector<std::string> thread_names;
+};
+
 auto const null_report = mr::null_compositor_report();
 unsigned int const composites_per_update{1};
 }
@@ -546,17 +580,25 @@ TEST(MultiThreadedCompositor, makes_and_releases_display_buffer_current_target)
 
 TEST(MultiThreadedCompositor, double_start_or_stop_ignored)
 {
+    using namespace testing;
+
     unsigned int const nbuffers{3};
     auto display = std::make_shared<StubDisplayWithMockBuffers>(nbuffers);
     auto mock_scene = std::make_shared<mtd::MockScene>();
     auto db_compositor_factory = std::make_shared<NullDisplayBufferCompositorFactory>();
     auto mock_report = std::make_shared<testing::NiceMock<mtd::MockCompositorReport>>();
+
     EXPECT_CALL(*mock_report, started())
         .Times(1);
     EXPECT_CALL(*mock_report, stopped())
         .Times(1);
-    EXPECT_CALL(*mock_scene, set_change_callback(testing::_))
-        .Times(2);
+    EXPECT_CALL(*mock_scene, add_observer(_))
+        .Times(1);
+    EXPECT_CALL(*mock_scene, remove_observer(_))
+        .Times(1);
+    EXPECT_CALL(*mock_scene, scene_elements_for(_))
+        .Times(AtLeast(0))
+        .WillRepeatedly(Return(mc::SceneElementSequence{}));
 
     mc::MultiThreadedCompositor compositor{display, mock_scene, db_compositor_factory, mock_report, true};
 
@@ -565,3 +607,77 @@ TEST(MultiThreadedCompositor, double_start_or_stop_ignored)
     compositor.stop();
     compositor.stop();
 }
+
+TEST(MultiThreadedCompositor, cleans_up_after_throw_in_start)
+{
+    unsigned int const nbuffers{3};
+
+    auto display = std::make_shared<StubDisplayWithMockBuffers>(nbuffers);
+    auto scene = std::make_shared<StubScene>();
+    auto db_compositor_factory = std::make_shared<RecordingDisplayBufferCompositorFactory>();
+    mc::MultiThreadedCompositor compositor{display, scene, db_compositor_factory, null_report, true};
+
+    scene->throw_on_add_observer(true);
+
+    EXPECT_THROW(compositor.start(), std::runtime_error);
+
+    scene->throw_on_add_observer(false);
+
+    /* No point in running the rest of the test if it throws again */
+    ASSERT_NO_THROW(compositor.start());
+
+    /* The minimum number of records here should be nbuffers *2, since we are checking for
+     * presence of at least one additional rogue compositor thread per display buffer
+     * However to avoid timing considerations like one good thread compositing the display buffer
+     * twice before the rogue thread gets a chance to, an arbitrary number of records are gathered
+     */
+    unsigned int min_number_of_records = 100;
+
+    /* Timeout here in case the exception from setting the scene callback put the compositor
+     * in a bad state that did not allow it to composite (hence no records gathered)
+     */
+    auto time_out = std::chrono::steady_clock::now() + std::chrono::seconds(1);
+    while (!db_compositor_factory->enough_records_gathered(nbuffers, min_number_of_records) &&
+           std::chrono::steady_clock::now() <= time_out)
+    {
+        scene->emit_change_event();
+        std::this_thread::yield();
+    }
+
+    /* Check expectation in case a timeout happened */
+    EXPECT_TRUE(db_compositor_factory->enough_records_gathered(nbuffers, min_number_of_records));
+
+    compositor.stop();
+
+    /* Only one thread should be rendering each display buffer
+     * If the compositor failed to cleanup correctly more than one thread could be
+     * compositing the same display buffer
+     */
+    EXPECT_TRUE(db_compositor_factory->each_buffer_rendered_in_single_thread());
+}
+
+TEST(MultiThreadedCompositor, names_compositor_threads)
+{
+    using namespace testing;
+
+    unsigned int const nbuffers{3};
+
+    auto display = std::make_shared<StubDisplayWithMockBuffers>(nbuffers);
+    auto scene = std::make_shared<StubScene>();
+    auto db_compositor_factory = std::make_shared<ThreadNameDisplayBufferCompositorFactory>();
+    mc::MultiThreadedCompositor compositor{display, scene, db_compositor_factory, null_report, true};
+
+    compositor.start();
+
+    unsigned int const min_number_of_thread_names = 10;
+
+    while (db_compositor_factory->num_thread_names_gathered() < min_number_of_thread_names)
+        scene->emit_change_event();
+
+    compositor.stop();
+
+    auto const& thread_names = db_compositor_factory->thread_names;
+
+    for (size_t i = 0; i < thread_names.size(); ++i)
+        EXPECT_THAT(thread_names[i], Eq("Mir/Comp")) << "i=" << i;
+}
