@@ -18,7 +18,7 @@
  */
 
 #include "mir/frontend/connector.h"
-#include "mir/frontend/resource_cache.h"
+#include "src/server/frontend/resource_cache.h"
 
 #include "mir_test/stub_server_tool.h"
 #include "mir_test/test_protobuf_server.h"
@@ -72,15 +72,6 @@ struct ErrorServer : StubServerTool
     {
         throw std::runtime_error(test_exception_text);
     }
-
-    void test_file_descriptors(
-        google::protobuf::RpcController*,
-        const protobuf::Void*,
-        protobuf::Buffer*,
-        google::protobuf::Closure*)
-    {
-        throw std::runtime_error(test_exception_text);
-    }
 };
 
 std::string const ErrorServer::test_exception_text{"test exception text"};
