@@ -23,7 +23,7 @@
 #include "src/server/frontend/resource_cache.h"
 #include "src/server/frontend/published_socket_connector.h"
 
-#include "mir/frontend/protobuf_session_creator.h"
+#include "mir/frontend/protobuf_connection_creator.h"
 #include "mir/frontend/connector_report.h"
 
 #include "mir_protobuf.pb.h"
@@ -35,6 +35,7 @@
 #include "mir_test/test_protobuf_client.h"
 #include "mir_test/test_protobuf_server.h"
 #include "mir_test_doubles/stub_ipc_factory.h"
+#include "mir_test_framework/testing_server_configuration.h"
 
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
@@ -47,6 +48,7 @@ namespace mf = mir::frontend;
 namespace mt = mir::test;
 namespace mtd = mir::test::doubles;
 namespace mr = mir::report;
+namespace mtf = mir_test_framework;
 
 namespace
 {
@@ -86,10 +88,10 @@ public:
 
 struct PublishedSocketConnector : public ::testing::Test
 {
-    static const char* const test_socket;
+    static std::string const test_socket;
     static void SetUpTestCase()
     {
-        remove(test_socket);
+        remove(test_socket.c_str());
     }
 
     void SetUp()
@@ -126,7 +128,8 @@ struct PublishedSocketConnector : public ::testing::Test
     static std::shared_ptr<mt::TestProtobufServer> stub_server;
 };
 
-const char* const PublishedSocketConnector::test_socket = "./test_socket";
+std::string const PublishedSocketConnector::test_socket = mtf::test_socket_file();
+
 std::shared_ptr<mt::StubServerTool> PublishedSocketConnector::stub_server_tool;
 std::shared_ptr<MockConnectorReport> PublishedSocketConnector::communicator_report;
 std::shared_ptr<mt::TestProtobufServer> PublishedSocketConnector::stub_server;
