@@ -19,14 +19,16 @@
 #include "mir_test_doubles/mock_fb_hal_device.h"
 #include "mir_test_doubles/mock_buffer.h"
 #include "src/platform/graphics/android/fb_device.h"
+#include "src/platform/graphics/android/hwc_fallback_gl_renderer.h"
 #include "mir_test_doubles/mock_framebuffer_bundle.h"
 #include "mir_test_doubles/mock_android_hw.h"
 #include "mir_test_doubles/mock_egl.h"
 #include "mir_test_doubles/stub_buffer.h"
 #include "mir_test_doubles/stub_renderable.h"
 #include "mir_test_doubles/mock_android_native_buffer.h"
-#include "mir_test_doubles/mock_render_function.h"
 #include "mir_test_doubles/mock_swapping_gl_context.h"
+#include "mir_test_doubles/stub_renderable_list_compositor.h"
+#include "mir_test_doubles/mock_renderable_list_compositor.h"
 
 #include <gtest/gtest.h>
 #include <stdexcept>
@@ -53,6 +55,8 @@ struct FBDevice : public ::testing::Test
         native_buffer = std::make_shared<mtd::StubAndroidNativeBuffer>();
         ON_CALL(*mock_buffer, native_buffer_handle())
             .WillByDefault(Return(native_buffer));
+        ON_CALL(mock_context, last_rendered_buffer())
+            .WillByDefault(Return(mock_buffer));
     }
 
     unsigned int width, height, format, fbnum;
@@ -60,33 +64,20 @@ struct FBDevice : public ::testing::Test
     std::shared_ptr<mtd::MockBuffer> mock_buffer;
     std::shared_ptr<mir::graphics::NativeBuffer> native_buffer;
     mtd::HardwareAccessMock hw_access_mock;
-    mtd::MockSwappingGLContext mock_context;
+    testing::NiceMock<mtd::MockSwappingGLContext> mock_context;
 };
 
-TEST_F(FBDevice, render_overlays_via_gl)
+TEST_F(FBDevice, rejects_overlays)
 {
-    auto renderable1 = std::make_shared<mtd::StubRenderable>();
-    auto renderable2 = std::make_shared<mtd::StubRenderable>();
     std::list<std::shared_ptr<mg::Renderable>> renderlist
     {
-        renderable1,
-        renderable2
+        std::make_shared<mtd::StubRenderable>(),
+        std::make_shared<mtd::StubRenderable>()
     };
 
-    mtd::MockRenderFunction mock_call_counter;
-    testing::Sequence seq;
-    EXPECT_CALL(mock_call_counter, called(testing::Ref(*renderable1)))
-        .InSequence(seq);
-    EXPECT_CALL(mock_call_counter, called(testing::Ref(*renderable2)))
-        .InSequence(seq);
-    EXPECT_CALL(mock_context, swap_buffers())
-        .InSequence(seq);
-
+    mtd::MockRenderableListCompositor stub_compositor;
     mga::FBDevice fbdev(fb_hal_mock);
-    fbdev.render_gl_and_overlays(mock_context, renderlist, [&](mg::Renderable const& renderable)
-    {
-        mock_call_counter.called(renderable);
-    });
+    EXPECT_FALSE(fbdev.post_overlays(mock_context, renderlist, stub_compositor));
 }
 
 TEST_F(FBDevice, commits_frame_via_post)
@@ -100,10 +91,10 @@ TEST_F(FBDevice, commits_frame_via_post)
     mga::FBDevice fbdev(fb_hal_mock);
 
     EXPECT_THROW({
-        fbdev.post(*mock_buffer);
+        fbdev.post_gl(mock_context);
     }, std::runtime_error);
 
-    fbdev.post(*mock_buffer);
+    fbdev.post_gl(mock_context);
 }
 
 TEST_F(FBDevice, sets_swapinterval_1_on_start)
