@@ -26,21 +26,26 @@ pushd ${BUILD_DIR} > /dev/null
     adb wait-for-device
     adb root
     adb wait-for-device
-    adb shell mkdir -p ${RUN_DIR}
+    adb shell mkdir -p ${RUN_DIR}/udev_recordings
 
     for x in bin/mir_acceptance_tests \
              bin/mir_integration_tests \
              bin/mir_unit_tests \
              lib/libmirclient.so.* \
-             lib/libmirprotobuf.so.* \
-             lib/libmirplatform.so \
-             lib/libmirplatformgraphics.so \
-             lib/libmirclientplatform.so \
+             lib/libmircommon.so.* \
+             lib/libmirplatform.so.* \
+             lib/libmirplatform*driver.so \
+             lib/libmirclient*driver.so \
              lib/libmirserver.so.*
     do
         adb push $x ${RUN_DIR}
     done
 
+    for x in bin/udev_recordings/*
+    do 
+        adb push $x ${RUN_DIR}/udev_recordings
+    done
+
     echo "cd ${RUN_DIR};
         export GTEST_OUTPUT=xml:./;
         export LD_LIBRARY_PATH=.;
